@@ -57,10 +57,10 @@ class Node(object):
         self.source_line = source_line
         self.source_column = source_column
 
-    if str is bytes:
+    if str is bytes:  # pragma: no cover
         def __repr__(self):
             return self.repr_format.format(self=self).encode('utf8')
-    else:
+    else:  # pragma: no cover
         def __repr__(self):
             return self.repr_format.format(self=self)
 
@@ -75,7 +75,7 @@ class Node(object):
         by calling the provided :obj:`write` callback.
 
         """
-        raise NotImplementedError
+        raise NotImplementedError  # pragma: no cover
 
 
 class ParseError(Node):
@@ -116,22 +116,24 @@ class ParseError(Node):
             write('url([bad url])')
         elif self.kind in ')]}':
             write(self.kind)
-        else:
+        else:  # pragma: no cover
             raise TypeError('Can not serialize %r' % self)
 
 
 class Comment(Node):
     """A CSS comment.
 
-    By default, comments are ignored
-    and :class:`Comment` objects are not created.
-    This can be changed by passing ``preserve_comments=True``
-    to :func:`~tinycss2.parse_component_value_list`
+    .. code-block:: text
+
+        '/*' <value> '*/'
+
+    Comments can be ignored by passing ``skip_comments=True``
+    to functions such as :func:`~tinycss2.parse_component_value_list`.
 
     .. autoattribute:: type
     .. attribute:: value
 
-        The content of the comment, between ``/*`` and ``*/``.
+        The content of the comment, between ``/*`` and ``*/``, as a string.
 
     """
     __slots__ = ['value']
@@ -152,14 +154,22 @@ class WhitespaceToken(Node):
     """A :diagram:`whitespace-token`.
 
     .. autoattribute:: type
+    .. attribute:: value
+
+        The whitespace sequence, as a string, as in the original CSS source.
+
 
     """
-    __slots__ = []
+    __slots__ = ['value']
     type = 'whitespace'
     repr_format = '<{self.__class__.__name__}>'
 
+    def __init__(self, line, column, value):
+        Node.__init__(self, line, column)
+        self.value = value
+
     def _serialize_to(self, write):
-        write(' ')
+        write(self.value)
 
 
 class LiteralToken(Node):
@@ -247,6 +257,10 @@ class IdentToken(Node):
 class AtKeywordToken(Node):
     """An :diagram:`at-keyword-token`.
 
+    .. code-block:: text
+
+        '@' <value>
+
     .. autoattribute:: type
 
     .. attribute:: value
@@ -261,7 +275,7 @@ class AtKeywordToken(Node):
 
         .. code-block:: python
 
-            if node.type == 'at-keyword' and node.lower_value == 'import': ...
+            if node.type == 'at-keyword' and node.lower_value == 'import':
 
     """
     __slots__ = ['value', 'lower_value']
@@ -281,6 +295,10 @@ class AtKeywordToken(Node):
 class HashToken(Node):
     r"""A :diagram:`hash-token`.
 
+    .. code-block:: text
+
+        '#' <value>
+
     .. autoattribute:: type
 
     .. attribute:: value
@@ -314,6 +332,10 @@ class HashToken(Node):
 class StringToken(Node):
     """A :diagram:`string-token`.
 
+    .. code-block:: text
+
+        '"' <value> '"'
+
     .. autoattribute:: type
 
     .. attribute:: value
@@ -338,6 +360,10 @@ class StringToken(Node):
 class URLToken(Node):
     """An :diagram:`url-token`.
 
+    .. code-block:: text
+
+        'url("' <value> '")'
+
     .. autoattribute:: type
 
     .. attribute:: value
@@ -432,6 +458,10 @@ class NumberToken(Node):
 class PercentageToken(Node):
     """A :diagram:`percentage-token`.
 
+    .. code-block:: text
+
+        <representation> '%'
+
     .. autoattribute:: type
 
     .. attribute:: value
@@ -473,6 +503,10 @@ class PercentageToken(Node):
 class DimensionToken(Node):
     """A :diagram:`dimension-token`.
 
+    .. code-block:: text
+
+        <representation> <unit>
+
     .. autoattribute:: type
 
     .. attribute:: value
@@ -506,7 +540,7 @@ class DimensionToken(Node):
 
         .. code-block:: python
 
-            if node.type == 'dimension' and node.lower_unit == 'px': ...
+            if node.type == 'dimension' and node.lower_unit == 'px':
 
     """
     __slots__ = ['value', 'int_value', 'is_integer', 'representation',
@@ -538,6 +572,10 @@ class DimensionToken(Node):
 class ParenthesesBlock(Node):
     """A :diagram:`()-block`.
 
+    .. code-block:: text
+
+        '(' <content> ')'
+
     .. autoattribute:: type
 
     .. attribute:: content
@@ -563,6 +601,10 @@ class ParenthesesBlock(Node):
 class SquareBracketsBlock(Node):
     """A :diagram:`[]-block`.
 
+    .. code-block:: text
+
+        '[' <content> ']'
+
     .. autoattribute:: type
 
     .. attribute:: content
@@ -588,6 +630,10 @@ class SquareBracketsBlock(Node):
 class CurlyBracketsBlock(Node):
     """A :diagram:`{}-block`.
 
+    .. code-block:: text
+
+        '{' <content> '}'
+
     .. autoattribute:: type
 
     .. attribute:: content
@@ -613,6 +659,10 @@ class CurlyBracketsBlock(Node):
 class FunctionBlock(Node):
     """A :diagram:`function-block`.
 
+    .. code-block:: text
+
+        <name> '(' <arguments> ')'
+
     .. autoattribute:: type
 
     .. attribute:: name
@@ -653,6 +703,11 @@ class FunctionBlock(Node):
 class Declaration(Node):
     """A (property or descriptor) :diagram:`declaration`.
 
+    .. code-block:: text
+
+        <name> ':' <value>
+        <name> ':' <value> '!important'
+
     .. autoattribute:: type
 
     .. attribute:: name
@@ -670,7 +725,7 @@ class Declaration(Node):
 
         .. code-block:: python
 
-            if node.type == 'declaration' and node.lower_name == 'color': ...
+            if node.type == 'declaration' and node.lower_name == 'color':
 
     .. attribute:: value
 
@@ -700,17 +755,22 @@ class Declaration(Node):
         write(serialize_identifier(self.name))
         write(':')
         _serialize_to(self.value, write)
-        write(';')
+        if self.important:
+            write('!important')
 
 
 class QualifiedRule(Node):
     """A :diagram:`qualified rule`.
 
-    The interpretation of style rules depend on their context.
+    .. code-block:: text
+
+        <prelude> '{' <content> '}'
+
+    The interpretation of qualified rules depend on their context.
     At the top-level of a stylesheet
     or in a conditional rule such as ``@media``,
-    they are style rules where the :attr:`prelude` is a list of Selectors
-    and the :attr:`body` is a list of property declarations.
+    they are **style rules** where the :attr:`prelude` is Selectors list
+    and the :attr:`content` is a list of property declarations.
 
     .. autoattribute:: type
 
@@ -745,6 +805,11 @@ class QualifiedRule(Node):
 class AtRule(Node):
     """An :diagram:`at-rule`.
 
+    .. code-block:: text
+
+        @<at_keyword> <prelude> '{' <content> '}'
+        @<at_keyword> <prelude> ';'
+
     The interpretation of at-rules depend on their at-keyword
     as well as their context.
     Most types of at-rules (ie. at-keyword values)
@@ -766,7 +831,7 @@ class AtRule(Node):
 
         .. code-block:: python
 
-            if node.type == 'at-rule' and node.lower_at_keyword == 'import': ...
+            if node.type == 'at-rule' and node.lower_at_keyword == 'import':
 
     .. attribute:: prelude
 
