@@ -1,6 +1,5 @@
 from webencodings import lookup, decode, UTF8
 
-from .tokenizer import parse_component_value_list
 from .parser import parse_stylesheet
 
 
@@ -47,33 +46,48 @@ def decode_stylesheet_bytes(css_bytes, protocol_encoding=None,
 
 
 def parse_stylesheet_bytes(css_bytes, protocol_encoding=None,
-                           environment_encoding=None):
-    """Parse :diagram:`stylesheet` from bytes.
+                           environment_encoding=None,
+                           skip_comments=False, skip_whitespace=False):
+    """Parse :diagram:`stylesheet` from bytes,
+    determining the character encoding as web browsers do.
 
     This is used when reading a file or fetching an URL.
     The character encoding is determined from the initial bytes
     (a :abbr:`BOM (Byte Order Mark)` or an ``@charset`` rule)
     as well as the parameters.
+    The ultimate fallback is UTF-8.
 
     :param css_bytes: A byte string.
     :param protocol_encoding:
+        A string.
         The encoding label, if any, defined by HTTP or equivalent protocol.
         (e.g. via the ``charset`` parameter of the ``Content-Type`` header.)
     :param environment_encoding:
         A :class:`webencodings.Encoding` object
         for the `environment encoding`_,
         if any.
+    :param skip_comments:
+        Ignore CSS comments at the top-level of the stylesheet.
+        If the input is a string, ignore all comments.
+    :param skip_whitespace:
+        Ignore whitespace at the top-level of the stylesheet.
+        Whitespace is still preserved
+        in the :attr:`~tinycss2.ast.QualifiedRule.prelude`
+        and the :attr:`~tinycss2.ast.QualifiedRule.content` of rules.
     :returns:
         A ``(rules, encoding)`` tuple.
 
         * :obj:`rules` is a list of
           :class:`~tinycss2.ast.QualifiedRule`,
           :class:`~tinycss2.ast.AtRule`,
+          :class:`~tinycss2.ast.Comment` (if ``skip_comments`` is false),
+          :class:`~tinycss2.ast.WhitespaceToken`
+          (if ``skip_whitespace`` is false),
           and :class:`~tinycss2.ast.ParseError` objects.
         * :obj:`encoding` is the :class:`webencodings.Encoding` object
-          that was used,
-          the `environment encoding`_ for stylesheets imported
-          from ``@import`` rules in :obj:`rules`.
+          that was used.
+          If ``rules`` contains an ``@import`` rule, this is
+          the `environment encoding`_ for the imported stylesheet.
 
     .. _environment encoding: http://www.w3.org/TR/css-syntax/#environment-encoding
 
@@ -93,4 +107,5 @@ def parse_stylesheet_bytes(css_bytes, protocol_encoding=None,
     """
     css_unicode, encoding = decode_stylesheet_bytes(
         css_bytes, protocol_encoding, environment_encoding)
-    return parse_stylesheet(parse_component_value_list(css_unicode)), encoding
+    stylesheet = parse_stylesheet(css_unicode, skip_comments, skip_whitespace)
+    return stylesheet, encoding
