@@ -44,11 +44,12 @@ def parse_color(input):
           (including keywords, HSL and HSLA.)
           The alpha channel is clipped to [0, 1]
           but red, green, or blue can be out of range
-          (eg. ``rgb(-10%, 120%, 0%)`` is represented as ``(-0.1, 1.2, 0, 1)``.)
+          (eg. ``rgb(-10%, 120%, 0%)`` is represented as
+          ``(-0.1, 1.2, 0, 1)``.)
 
     """
     if isinstance(input, basestring):
-        token = parse_one_component_value(input)
+        token = parse_one_component_value(input, skip_comments=True)
     else:
         token = input
     if token.type == 'ident':
@@ -163,7 +164,8 @@ def _parse_comma_separated(tokens):
     or None if the function token content do not match the description above.
 
     """
-    tokens = [token for token in tokens if token.type != 'whitespace']
+    tokens = [token for token in tokens
+              if token.type not in ('whitespace', 'comment')]
     if not tokens:
         return []
     if len(tokens) % 2 == 1 and all(token == ',' for token in tokens[1::2]):
