@@ -5,32 +5,34 @@ from .ast import ParseError, Declaration, AtRule, QualifiedRule
 from ._compat import basestring
 
 
-def _to_token_iterator(input):
+def _to_token_iterator(input, skip_comments=False):
     """
 
     :param input: A string or an iterable of :term:`component values`.
+    :param skip_comments:
+        If the input is a string, ignore all CSS comments.
     :returns: A iterator yielding :term:`component values`.
 
     """
     # Accept ASCII-only byte strings on Python 2, with implicit conversion.
     if isinstance(input, basestring):
-        input = parse_component_value_list(input)
+        input = parse_component_value_list(input, skip_comments)
     return iter(input)
 
 
-def _next_non_whitespace(tokens):
-    """Return the next non-<whitespace> token.
+def _next_significant(tokens):
+    """Return the next significant (neither whitespace or comment) token.
 
     :param tokens: An *iterator* yielding :term:`component values`.
     :returns: A :term:`component value`, or :obj:`None`.
 
     """
     for token in tokens:
-        if token.type != 'whitespace':
+        if token.type not in ('whitespace', 'comment'):
             return token
 
 
-def parse_one_component_value(input):
+def parse_one_component_value(input, skip_comments=False):
     """Parse a single :diagram:`component value`.
 
     This is used e.g. for an attribute value
@@ -38,12 +40,16 @@ def parse_one_component_value(input):
 
     :param input:
         A :term:`string`, or an iterable of :term:`component values`.
-    :returns: A :term:`component value`, or a :class:`~tinycss2.ast.ParseError`.
+    :param skip_comments:
+        If the input is a string, ignore all CSS comments.
+    :returns:
+        A :term:`component value` (that is neither whitespace or comment),
+        or a :class:`~tinycss2.ast.ParseError`.
 
     """
-    tokens = _to_token_iterator(input)
-    first = _next_non_whitespace(tokens)
-    second = _next_non_whitespace(tokens)
+    tokens = _to_token_iterator(input, skip_comments)
+    first = _next_significant(tokens)
+    second = _next_significant(tokens)
     if first is None:
         return ParseError(1, 1, 'empty', 'Input is empty')
     if second is not None:
@@ -54,7 +60,7 @@ def parse_one_component_value(input):
         return first
 
 
-def parse_one_declaration(input):
+def parse_one_declaration(input, skip_comments=False):
     """Parse a single :diagram:`declaration`.
 
     This is used e.g. for a declaration in an `@supports
@@ -62,31 +68,28 @@ def parse_one_declaration(input):
 
     :param input:
         A :term:`string`, or an iterable of :term:`component values`.
+    :param skip_comments:
+        If the input is a string, ignore all CSS comments.
     :returns:
         A :class:`~tinycss2.ast.Declaration`
         or :class:`~tinycss2.ast.ParseError`.
 
+    Any whitespace or comment before the ``:`` colon is dropped.
+
     """
-    tokens = _to_token_iterator(input)
-    first_token = _next_non_whitespace(tokens)
+    tokens = _to_token_iterator(input, skip_comments)
+    first_token = _next_significant(tokens)
     if first_token is None:
         return ParseError(1, 1, 'empty', 'Input is empty')
-    result = _consume_declaration(first_token, tokens)
-    if result.type != 'error':
-        next = _next_non_whitespace(tokens)
-        if next is not None:
-            return ParseError(
-                next.source_line, next.source_column, 'extra-input',
-                'Expected a single rule, got %s after the first rule.'
-                % next.type)
-    return result
+    return _parse_declaration(first_token, tokens)
 
 
-def _consume_declaration(first_token, tokens):
+def _parse_declaration(first_token, tokens):
     """Parse a declaration.
 
     Consume :obj:`tokens` until the end of the declaration or the first error.
 
+    :param first_token: The first :term:`component value` of the rule.
     :param tokens: An *iterator* yielding :term:`component values`.
     :returns:
         A :class:`~tinycss2.ast.Declaration`
@@ -99,7 +102,7 @@ def _consume_declaration(first_token, tokens):
                           'Expected <ident> for declaration name, got %s.'
                           % name.type)
 
-    colon = _next_non_whitespace(tokens)
+    colon = _next_significant(tokens)
     if colon is None:
         return ParseError(name.source_line, name.source_column, 'invalid',
                           "Expected ':' after declaration name, got EOF")
@@ -109,70 +112,83 @@ def _consume_declaration(first_token, tokens):
                           % colon.type)
 
     value = []
-    important = False
+    state = 'value'
+    for i, token in enumerate(tokens):
+        if state == 'value' and token == '!':
+            state = 'bang'
+            bang_position = i
+        elif state == 'bang' and token.type == 'ident' \
+                and token.lower_value == 'important':
+            state = 'important'
+        elif token.type not in ('whitespace', 'comment'):
+            state = 'value'
+        value.append(token)
+
+    if state == 'important':
+        del value[bang_position:]
+
+    return Declaration(name.source_line, name.source_column, name.value,
+                       name.lower_value, value, state == 'important')
+
+
+def _consume_declaration_in_list(first_token, tokens):
+    """Like :func:`_parse_declaration`, but stop at the first ``;``."""
+    other_declaration_tokens = []
     for token in tokens:
         if token == ';':
             break
-        elif token == '!':
-            token = _next_non_whitespace(tokens)
-            if (token is not None and token.type == 'ident'
-                    and token.lower_value == 'important'):
-                token = _next_non_whitespace(tokens)
-                if token is None or token == ';':
-                    important = True
-                    break
-            return ParseError(
-                token.source_line, token.source_column, 'invalid',
-                "Invalid '!' value, expected 'important'.")
-        else:
-            value.append(token)
-
-    return Declaration(name.source_line, name.source_column, name.value,
-                       name.lower_value, value, important)
+        other_declaration_tokens.append(token)
+    return _parse_declaration(first_token, iter(other_declaration_tokens))
 
 
-def _consume_whole_declaration(first_token, tokens):
-    """
-    Same as :func:`_consume_declaration`, but consume to the end
-    of the (possibly invalid) declaration.
-
-    """
-    result = _consume_declaration(first_token, tokens)
-    if result.type == 'error':
-        # Consume until the next ';' or EOF.
-        for token in tokens:
-            if token == ';':
-                break
-    return result
-
-
-def parse_declaration_list(input):
+def parse_declaration_list(input, skip_comments=False, skip_whitespace=False):
     """Parse a :diagram:`declaration list` (which may also contain at-rules).
 
     This is used e.g. for the :attr:`~tinycss2.ast.QualifiedRule.content`
     of a style rule or ``@page`` rule,
-    or for the `style` attribute of an HTML element.
+    or for the ``style`` attribute of an HTML element.
 
     In contexts that don’t expect any at-rule,
     all :class:`~tinycss2.ast.AtRule` objects
     should simply be rejected as invalid.
 
     :param input: A string or an iterable of :term:`component values`.
+    :param skip_comments:
+        Ignore CSS comments at the top-level of the list.
+        If the input is a string, ignore all comments.
+    :param skip_whitespace:
+        Ignore whitespace at the top-level of the list.
+        Whitespace is still preserved
+        in the :attr:`~tinycss2.ast.Declaration.value` of declarations
+        and the :attr:`~tinycss2.ast.AtRule.prelude`
+        and :attr:`~tinycss2.ast.AtRule.content` of at-rules.
     :returns:
         A list of
         :class:`~tinycss2.ast.Declaration`,
         :class:`~tinycss2.ast.AtRule`,
+        :class:`~tinycss2.ast.Comment` (if ``skip_comments`` is false),
+        :class:`~tinycss2.ast.WhitespaceToken`
+        (if ``skip_whitespace`` is false),
         and :class:`~tinycss2.ast.ParseError` objects
 
     """
-    tokens = _to_token_iterator(input)
-    return [
-        _consume_at_rule(token, tokens) if token.type == 'at-keyword'
-        else _consume_whole_declaration(token, tokens)
-        for token in tokens if token.type != 'whitespace' and token != ';']
+    tokens = _to_token_iterator(input, skip_comments)
+    result = []
+    for token in tokens:
+        if token.type == 'whitespace':
+            if not skip_whitespace:
+                result.append(token)
+        elif token.type == 'comment':
+            if not skip_comments:
+                result.append(token)
+        elif token.type == 'at-keyword':
+            result.append(_consume_at_rule(token, tokens))
+        elif token != ';':
+            result.append(_consume_declaration_in_list(token, tokens))
+    return result
 
 
-def parse_one_rule(input):
+def parse_one_rule(input, skip_comments=False):
     """Parse a single :diagram:`qualified rule` or :diagram:`at-rule`.
 
     This would be used e.g. by `insertRule()
@@ -180,19 +196,23 @@ def parse_one_rule(input):
     in an implementation of CSSOM.
 
     :param input: A string or an iterable of :term:`component values`.
+    :param skip_comments:
+        If the input is a string, ignore all CSS comments.
     :returns:
         A :class:`~tinycss2.ast.QualifiedRule`,
         :class:`~tinycss2.ast.AtRule`,
         or :class:`~tinycss2.ast.ParseError` objects.
 
+    Any whitespace or comment before or after the rule is dropped.
+
     """
-    tokens = _to_token_iterator(input)
-    first = _next_non_whitespace(tokens)
+    tokens = _to_token_iterator(input, skip_comments)
+    first = _next_significant(tokens)
     if first is None:
         return ParseError(1, 1, 'empty', 'Input is empty')
 
     rule = _consume_rule(first, tokens)
-    next = _next_non_whitespace(tokens)
+    next = _next_significant(tokens)
     if next is not None:
         return ParseError(
             next.source_line, next.source_column, 'extra-input',
@@ -200,7 +220,7 @@ def parse_one_rule(input):
     return rule
 
 
-def parse_rule_list(input):
+def parse_rule_list(input, skip_comments=False, skip_whitespace=False):
     """Parse a non-top-level :diagram:`rule list`.
 
     This is used for parsing the :attr:`~tinycss2.ast.AtRule.content`
@@ -209,37 +229,78 @@ def parse_rule_list(input):
     top-level ``<!--`` and ``-->`` tokens are not ignored.
 
     :param input: A string or an iterable of :term:`component values`.
+    :param skip_comments:
+        Ignore CSS comments at the top-level of the list.
+        If the input is a string, ignore all comments.
+    :param skip_whitespace:
+        Ignore whitespace at the top-level of the list.
+        Whitespace is still preserved
+        in the :attr:`~tinycss2.ast.QualifiedRule.prelude`
+        and the :attr:`~tinycss2.ast.QualifiedRule.content` of rules.
     :returns:
         A list of
         :class:`~tinycss2.ast.QualifiedRule`,
         :class:`~tinycss2.ast.AtRule`,
+        :class:`~tinycss2.ast.Comment` (if ``skip_comments`` is false),
+        :class:`~tinycss2.ast.WhitespaceToken`
+        (if ``skip_whitespace`` is false),
         and :class:`~tinycss2.ast.ParseError` objects.
 
     """
-    tokens = _to_token_iterator(input)
-    return [_consume_rule(token, tokens) for token in tokens
-            if token.type != 'whitespace']
+    tokens = _to_token_iterator(input, skip_comments)
+    result = []
+    for token in tokens:
+        if token.type == 'whitespace':
+            if not skip_whitespace:
+                result.append(token)
+        elif token.type == 'comment':
+            if not skip_comments:
+                result.append(token)
+        else:
+            result.append(_consume_rule(token, tokens))
+    return result
 
 
-def parse_stylesheet(input):
+def parse_stylesheet(input, skip_comments=False, skip_whitespace=False):
     """Parse :diagram:`stylesheet` from text.
 
     This is used e.g. for a ``<style>`` HTML element.
 
     This differs from :func:`parse_rule_list` in that
     top-level ``<!--`` and ``-->`` tokens are ignored.
+    This is a legacy quirk for the ``<style>`` HTML element.
 
     :param input: A string or an iterable of :term:`component values`.
+    :param skip_comments:
+        Ignore CSS comments at the top-level of the stylesheet.
+        If the input is a string, ignore all comments.
+    :param skip_whitespace:
+        Ignore whitespace at the top-level of the stylesheet.
+        Whitespace is still preserved
+        in the :attr:`~tinycss2.ast.QualifiedRule.prelude`
+        and the :attr:`~tinycss2.ast.QualifiedRule.content` of rules.
     :returns:
         A list of
         :class:`~tinycss2.ast.QualifiedRule`,
         :class:`~tinycss2.ast.AtRule`,
+        :class:`~tinycss2.ast.Comment` (if ``skip_comments`` is false),
+        :class:`~tinycss2.ast.WhitespaceToken`
+        (if ``skip_whitespace`` is false),
         and :class:`~tinycss2.ast.ParseError` objects.
 
     """
-    tokens = _to_token_iterator(input)
-    return [_consume_rule(token, tokens) for token in tokens
-            if token.type != 'whitespace' and token not in ('<!--', '-->')]
+    tokens = _to_token_iterator(input, skip_comments)
+    result = []
+    for token in tokens:
+        if token.type == 'whitespace':
+            if not skip_whitespace:
+                result.append(token)
+        elif token.type == 'comment':
+            if not skip_comments:
+                result.append(token)
+        elif token not in ('<!--', '-->'):
+            result.append(_consume_rule(token, tokens))
+    return result
 
 
 def _consume_rule(first_token, tokens):
